#
# WiKicker
#
#  HTML.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: HTML.pm,v 1.21 2004/08/03 12:55:52 naney Exp $
#
#

package WiKicker::CGI::View::HTML;

use 5.005;
use strict;
use CGI qw/-no_xhtml/;
use WiKicker::CGI::View;
use WiKicker::Time;
use WiKicker::HTML;
use Time::HiRes qw(tv_interval);

use vars qw(@ISA);

BEGIN {
  @ISA = qw(WiKicker::CGI::View);
}

sub view {
  my $self = shift;

  my $response = $self->response;

  $self->_set_html_http_header;
  $response->flush;

  $self->make_start_html;
  $response->flush;

  $self->make_html_header;
  $response->flush;

  $self->make_html_body;
  $response->flush;

  $self->make_html_footer;
}

# override subclass
# For example, return '503 Unavailable';
sub http_status {
  return '';
}

sub _set_html_http_header {
  my $self = shift;

  my $response = $self->response;
  my %param;
  $param{-type} = 'text/html; charset=UTF-8';
  $param{-charset} = 'UTF-8';
  if (@{$response->{cookies}} > 0) {
    $param{-cookie} = $response->{cookies};
  }
  my $last_modified = $self->request->attribute('http_last_modified_for_send');
  if ($last_modified) {
    $param{-Last_Modified} = WiKicker::Time::gmtime_to_RFC1123($last_modified);
  }
  my $status = $self->http_status;
  if ($status ne '') {
    $param{-status} = $status;
  }

  $response->http_header($self->request->query->header(%param));
}


sub make_start_html {
  my $self = shift;

  #my $doctype = q(<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">);

  my $doctype = q(<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">);

  $self->response->add_content("$doctype\n");

  my $request = $self->request;
  my $lang = $request->lang;
  my $title = $self->make_html_head_title;

  $self->response->add_content(qq(<html lang="$lang">\n<head>\n));
  $self->response->add_content(qq(  <title>$title</title>\n));

  my @head = @{$self->_make_default_head};

  my @link_element_list = $self->make_link_element_list;
  if (@link_element_list) {
    push(@head, @link_element_list);
  }

  my @stylesheet_uri_list = $self->stylesheet_uri_list;
  if (@stylesheet_uri_list) {
    map {
      my $escaped_uri = WiKicker::HTML::escape_attribute($_);
      push(@head,
           qq(  <link rel="stylesheet" type="text/css" href="$escaped_uri">));
    } @stylesheet_uri_list;
  }
  my $external_scripts = $self->external_head_scripts_list;
  for my $item (@$external_scripts) {
    my $uri = $item->{uri};
    my $type = $item->{type};
    my $charset = $item->{charset};
    $uri = WiKicker::HTML::escape_attribute($uri);
    $type = WiKicker::HTML::escape_attribute($type);
    if ($charset ne '') {
      $charset = WiKicker::HTML::escape_attribute($charset);
      push(@head, qq(  <script type="$type" charset="$charset" src="$uri"></script>));
    }
    else {
      push(@head, qq(  <script type="$type" src="$uri"></script>));
    }
  }

  my $head_comments = $self->request->attribute('head_comments');
  if (defined $head_comments) {
    map { push(@head, qq(<!-- $_ -->\n)) } @$head_comments;
  }

  $self->response->add_content(join("\n", @head));
  $self->response->add_content("\n</head>\n<body>\n");
}

# for override.
sub make_html_header {
  my $self = shift;
}

# for override.
sub make_html_body {
  my $self = shift;
}

# for override.
sub make_html_footer {
  my $self = shift;

  $self->response->add_content('</body></html>');
}

sub make_link_element_list { return () };
sub make_html_head_title { '(title)' }
sub stylesheet_uri { '' }
sub external_head_scripts_list{ [] }

sub load_footer_file {
  my ($self, $property_name, $default_footer_file) = @_;

  unless ($default_footer_file =~ m|^/|) {
    $default_footer_file = '/' . $default_footer_file;
  }

  my @footer_files = map {$_ . $default_footer_file } @INC;

  my $user_footer_file = $self->request->configuration->properties->get($property_name);
  if (defined $user_footer_file && $user_footer_file ne '') {
    unshift(@footer_files, $user_footer_file);
  }

  my $footer = '';
  for my $path (@footer_files) {
    if (-e $path) {
      unless (open(FOOTER, $path)) {
        warn "Can't open footer: $path: $!";
        next;
      }
      else {
        local $/;
        $footer = <FOOTER>;
        close(FOOTER);
        last;
      }
    }
  }
  return $footer;
}

sub common_footer_text_filter {
  my ($self, $footer) = @_;

  my $load_average = $self->request->load_average->load_average_string;
  my $process_time = 'Process Time: ' . tv_interval($self->request->attribute('start_time')) . 's';

  $footer =~ s/\[\[value:LoadAverage\]\]/$load_average/g;
  $footer =~ s/\[\[value:ProcessTime\]\]/$process_time/g;
  $footer =~ s/\[\[value:Version\]\]/$WiKicker::VERSION/g;

  return $footer;
}

sub _make_default_head {
  my $self = shift;

  my $run_count = $self->request->attribute('run_count') || 0;

  return
    [
      "  <!-- WiKicker $WiKicker::VERSION on $^O Perl $] -->",
      "  <!-- Run: $$ ($run_count) -->",
      q(  <meta http-equiv="Content-Type" content="text/html;charset=UTF-8">),
      q(  <meta http-equiv="Content-Style-Type" content="text/css">),
    ];
}

1;
