#
# WiKicker
#
#  PrimaryDatabase.pm
#
#
# Copyright (C) 2004,2005 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: PrimaryDatabase.pm,v 1.5 2005/03/06 09:06:48 naney Exp $
#
#

package WiKicker::Diki::PrimaryDatabase;

use 5.005;
use strict;
use Carp;
use WiKicker::Diki::PrimaryYearDatabase;
use WiKicker::Diki::PrimaryKeywordDatabase;

sub new {
  my ($class, %param) = @_;

  unless (exists $param{directory}) {
    croak 'No directory';
  }
  unless (-d $param{directory}) {
    croak q(diary primary directory ') .
      $param{directory} . q(' is not directory (or is not accessible). Please make the directory.);
  }

  my $self = bless { directory => $param{directory} }, $class;
}

sub directory {
  shift->{directory};
}

sub diary_year_directory {
  my ($self, $year) = @_;

  return $self->directory . '/' . $year;
}

sub keyword_directory { shift->directory . '/keyword' }

sub years {
  my $self = shift;

  if ($self->{years}) {
    return $self->{years};
  }

  opendir(DIR, $self->directory) || croak "Can't open directory: $!";
  my @files;
  my $file;
  while (defined ($file = readdir(DIR))) {
    if ($file =~ /^\d\d\d\d$/) {
      push(@files, $file);
    }
  }
  closedir(DIR);

  $self->{years} = [ sort @files ];
}


sub recent_year {
  my $self = shift;

  my $years = $self->years;
  if (scalar(@$years) == 0) {
    return '';
  }
  else {
    return $years->[-1];
  }
}

sub year_db {
  my ($self, $year) = @_;

  if (exists $self->{year_db}->{$year}) {
    return $self->{year_db}->{$year};
  }

  return $self->{year_db}->{$year}
    = WiKicker::Diki::PrimaryYearDatabase
      ->new(directory => $self->diary_year_directory($year),
            year => $year);
}

sub first_year {
  my $self = shift;

  my $years = $self->years;
  if (scalar(@$years) == 0) {
    return '';
  }
  else {
    return $years->[0];
  }
}

sub keyword_db {
  my $self = shift;

  if (exists $self->{keyword_db}) {
    return $self->{keyword_db};
  }
  return $self->{keyword_db} = WiKicker::Diki::PrimaryKeywordDatabase
    ->new(directory => $self->keyword_directory);
}

# return [ [date, last_modified], ... ]
sub get_recent_diaries_day_and_time {
  my ($self, $num_of_days) = @_;

  my @result = ();
  my $recent_year = $self->recent_year;
  my $first_year  = $self->first_year;

  for (my $year = $recent_year; $year >= $first_year; $year--) {
    push(@result,
         $self->_get_recent_diaries_day_and_time_from($year,
                                                      $num_of_days - @result
                                                     ));
    return \@result if @result == $num_of_days;
  }

  return \@result;
}

# return [ [date, last_modified], ... ]
sub _get_recent_diaries_day_and_time_from {
  my ($self, $year, $max) = @_;

  my $file_name_list =
    $self->_get_recent_diary_files_in_year($year, $max);

  my @result = ();
  for my $file_name (@$file_name_list) {
    my $date = $self->_file_name_to_date($file_name);
    my @stat = stat($file_name);
    my $last_modified = $stat[9];
    push(@result, [$date, $last_modified]);
  }

  return \@result;
}

sub check_diaries {
  my ($self, $diary_day_list) = @_;

  return undef;
}

sub check_recent_diaries {
  my ($self, $diary_day_list, $num_of_days) = @_;

  my $primary_article_set = $self->_recent_diary_article_set($num_of_days);
  my $secondary_article_set = $diary_day_list->article_last_modified_set;

  return $self->_check_primary_and_secondary($primary_article_set,
                                             $secondary_article_set);
}

sub check_article_diaries {
  my ($self, $diary_day_list, $range) = @_;

  my @result;
  my $primary_article_set = $self->_article_diary_article_set($range);
  my $secondary_article_set = $diary_day_list->article_last_modified_set;

  return $self->_check_primary_and_secondary($primary_article_set,
                                             $secondary_article_set);
}

sub check_one_day_diaries {
  my ($self, $diary_day_list, $range) = @_;

  my @result;
  my $primary_article_set = $self->_one_day_diary_article_set($range);
  my $secondary_article_set = $diary_day_list->article_last_modified_set;

  return $self->_check_primary_and_secondary($primary_article_set,
                                             $secondary_article_set);
}

sub check_ten_day_diaries {
  my ($self, $diary_day_list, $range) = @_;

  my @result;
  my $primary_article_set = $self->_ten_day_diary_article_set($range);
  my $secondary_article_set = $diary_day_list->article_last_modified_set;

  return $self->_check_primary_and_secondary($primary_article_set,
                                             $secondary_article_set);
}


sub _check_primary_and_secondary {
  my ($self, $primary_article_set, $secondary_article_set) = @_;

  my @result;
  for my $article_id (keys %$primary_article_set) {
    if (exists $secondary_article_set->{$article_id}) {
      if ($primary_article_set->{$article_id}
          != $secondary_article_set->{$article_id}) {
        my $article = $self->_get_diary_article($article_id);
        push(@result,
             bless [ $article_id, 'updated', $article,
                     $primary_article_set->{$article_id} ],
             'WiKicker::Diki::PrimaryDatabase::DiffItem'
            );
      }
    }
    else {
      my $article = $self->_get_diary_article($article_id);
      push(@result,
           bless [ $article_id, 'new', $article,
                   $primary_article_set->{$article_id} ],
             'WiKicker::Diki::PrimaryDatabase::DiffItem'
          );
    }
  }

  for my $article_id (keys %$secondary_article_set) {
    unless (exists $primary_article_set->{$article_id}) {
      push(@result,
           bless [ $article_id, 'deleted' ],
           'WiKicker::Diki::PrimaryDatabase::DiffItem'
          );
    }
  }

  if (@result) {
    return \@result;
  }
  else {
    return undef;
  }
}

sub check_keywords {
  my ($self, $keyword_list) = @_;

  return $self->keyword_db->check_keywords($keyword_list);
}

sub _recent_diary_article_set {
  my ($self, $num_of_days) = @_;

  my $rest_days = $num_of_days;
  my %result;

  for my $year(reverse @{$self->years}) {
    my $year_db = $self->year_db($year);
    my ($day_count, $set) = $year_db->recent_diary_article_set($rest_days);
    $rest_days -= $day_count;
    for my $article_id (keys %$set) {
      $result{$article_id} = $set->{$article_id};
    }
    last if $rest_days <= 0;
  }

  return \%result;
}

sub _article_diary_article_set {
  my ($self, $range) = @_;

  $range =~ /^(\d\d\d\d)-/;
  return $self->year_db($1)->article_diary_article_set($range);
}

sub _one_day_diary_article_set {
  my ($self, $range) = @_;

  $range =~ /^(\d\d\d\d)-/;
  return $self->year_db($1)->one_day_diary_article_set($range);
}

sub _ten_day_diary_article_set {
  my ($self, $range) = @_;

  $range =~ /^(\d\d\d\d)-/;
  return $self->year_db($1)->ten_day_diary_article_set($range);
}

sub _get_diary_article {
  my ($self, $article_id) = @_;

  $article_id =~ /^(\d\d\d\d)/;
  return $self->year_db($1)->get_article($article_id);
}

sub keyword_id_modified_iterator {
  my $self = shift;

  $self->keyword_db->keyword_id_modified_iterator;
}

sub get_keyword_article {
  my ($self, $keyword_id) = @_;

  $self->keyword_db->get_keyword_article($keyword_id);
}

sub ten_day_list {
  my $self = shift;

  my @result;
  for my $year (@{$self->years}) {
    my $year_db = $self->year_db($year);
    push(@result, @{$year_db->ten_day_list});
  }

  return \@result;
}

1;

package WiKicker::Diki::PrimaryDatabase::DiffItem;

sub article_id    { shift->[0] }
sub diff_type     { shift->[1] }
sub text          { shift->[2] }
sub last_modified { shift->[3] }

1;
