#
# WiKicker
#
#  WordExtractor.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: WordExtractor.pm,v 1.14 2004/08/03 12:55:52 naney Exp $
#
#

package WiKicker::Text::WordExtractor;

use strict;
use WiKicker::UTF8;
use WiKicker::Tree::OnDemandPageNameTrie;

sub new {
  my $class = shift;

  my $self = bless {}, $class;

  return $self;
}

sub DESTROY {
  my $self = shift;

  #-IFDEBUG: WiKicker::cycle_check($self);
  #-IFDEBUG: WiKicker::total_size("WordExtractor(before DESTROY)", $self);
}

sub set_words {
  my ($self, $words_list_ref, $enable_wiki_name) = @_;

  my $trie = WiKicker::Tree::OnDemandPageNameTrie->new;
  map { $trie->add($_, $enable_wiki_name) } @$words_list_ref;

  $self->{trie} = $trie;
}

sub split_into_words {
  my ($self, $text) = @_;

  my $c_list = WiKicker::UTF8::split_into_array_ref($text);
  my @suffix_array;
  $#suffix_array = scalar(@$c_list) - 1;
  for (my $i = scalar(@$c_list) - 1; $i >= 0; $i--) {
    if ($self->_right_connectable($c_list, $i)) {
      $suffix_array[$i] = [];
    }
    else {
      $suffix_array[$i]
        = WiKicker::Tree::OnDemandPageNameTrie::lookup($c_list, 0, $i,
                                                       $self->{trie});
    }
  }

  $self->_split_into_words(\@suffix_array, 0, @suffix_array - 1);
  return $self->_make_result($c_list, \@suffix_array);
}

sub _right_connectable  {
  my ($self, $c_list_ref, $pos) = @_;

  if ($pos == @$c_list_ref - 1) {
    return 0;
  }

  my $last_char = $c_list_ref->[$pos];
  my $next_char = $c_list_ref->[$pos + 1];

  return 1 if $last_char eq ' ';
  return 0 if $next_char eq ' ';
  return 1 if $last_char =~ /[a-z]/o && $next_char =~ /[a-z]/o;
  return 1 if $last_char =~ /[0-9]/o && $next_char =~ /[0-9]/o;

  return 0;
}

# sub _debug_print {
#   my ($self, $suffix_array_ref, $message) = @_;
#   print $message, '-' x 20, "\n";
#   for my $item (@$suffix_array_ref) {
#     print '[';
#     for my $suffix (@$item) {
#       print ' [' , $suffix->[0], ', ', (defined $suffix->[1] ? $suffix->[1] : '-' ), '] ';
#     }
#     print "]\n";
#   }
#   print '-' x 20, "\n";
# }

sub _split_into_words {
  my ($self, $suffix_array_ref, $first, $last) = @_;

  #$self->_debug_print($suffix_array_ref, "[$first, $last]");

  my ($max_suffix_position, $max_suffix_length)
    = $self->_get_max_suffix($suffix_array_ref, $first, $last);

  # no word?
  if ($max_suffix_position != -1) {
    # clear in words
    $self->_clear_in_suffix($suffix_array_ref, $first, $last,
                            $max_suffix_position, $max_suffix_length);
    my $left_last = $max_suffix_position - $max_suffix_length - 1;
    my $right_first = $max_suffix_position + 2;

    if ($left_last >= $first) {
      $self->_clear_overlap($suffix_array_ref, $first, $left_last);
      $self->_split_into_words($suffix_array_ref, $first, $left_last);
    }
    if ($right_first <= $last) {
      $self->_clear_overlap($suffix_array_ref, $right_first, $last);
      $self->_split_into_words($suffix_array_ref, $right_first, $last);
    }
  }
}

sub _clear_overlap {
  my ($self, $suffix_array_ref, $first, $last) = @_;

  for (my $i = $first; $i <= $last; $i++) {
    my $item = $suffix_array_ref->[$i];
    while (@$item > 0) {
      if ($i - $item->[0]->[0] + 1 < $first) {
        shift(@$item);
      }
      else {
        last;
      }
    }
  }
}

sub _get_max_suffix {
  my ($self, $suffix_array_ref, $first, $last) = @_;
  my $max_suffix_length = 0;
  my $max_suffix_position = -1;
  for (my $i = $first; $i <= $last; $i++) {
    my $suffix_length = -1;
    my $item = $suffix_array_ref->[$i];
    if (@$item > 0) {
      $suffix_length = $item->[0]->[0];
    }
    if ($suffix_length > $max_suffix_length) {
      $max_suffix_length = $suffix_length;
      $max_suffix_position = $i;
    }
  }
  return ($max_suffix_position, $max_suffix_length);
}

sub _clear_in_suffix {
  my ($self, $suffix_array_ref, $first, $last,
      $max_suffix_position, $max_suffix_length) = @_;

  $suffix_array_ref->[$max_suffix_position + 1] = []
    if $max_suffix_position < $last;

  my $clear_first_position = $max_suffix_position - $max_suffix_length;
  $clear_first_position = $first if $clear_first_position < $first;
  for (my $i = $max_suffix_position - 1; $i >= $clear_first_position; $i--) {
    $suffix_array_ref->[$i] = [];
  }
}

sub _make_result {
  my ($self, $c_list_ref, $suffix_array_ref) = @_;

  my $i = @$c_list_ref - 1;
  my $no_word_last = $i;
  my @result;
  while ($i >= 0) {
    my $item = $suffix_array_ref->[$i];
    if (@$item > 0) {
      if ($no_word_last > $i) {
        unshift(@result,
                [0, join('', @{$c_list_ref}[$i + 1..$no_word_last]), undef]);
      }
      unshift(@result,
              [1, join('', @{$c_list_ref}[$i - $item->[0]->[0] + 1 .. $i]), $item->[0]->[1]]);
      $i -= $item->[0]->[0];
      $no_word_last = $i;
    }
    else {
      $i--;
    }
  }
  if ($no_word_last > $i) {
    unshift(@result, [0, join('', @{$c_list_ref}[0 .. $no_word_last]), undef]);
  }

  return \@result;
}


1;
