#
# WiKicker
#
#  Rcs.pm
#
#
# Copyright (C) 2003,2004,2005 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Rcs.pm,v 1.5 2005/05/06 13:08:34 naney Exp $
#
#
package WiKicker::Tie::Rcs;

use 5.005;
use strict;
use File::stat;
use Time::Local;
use File::Spec;
use WiKicker::Tie::File;
use Fcntl ':flock';

use vars qw(@ISA);
@ISA = qw(WiKicker::Tie::File);

sub new {
  my ($class, $directory, $options) = @_;

  unless (exists $options->{check_in_interval}) {
    $options->{check_in_interval} = 60 * 15;
  }

  return WiKicker::Tie::File::new($class, $directory, $options);
}

sub _initialize {
  my $self = shift;

  $self->WiKicker::Tie::File::_initialize;

  my $rcs_dir = File::Spec->catfile($self->{directory}, 'RCS');
  unless (-d $rcs_dir) {
    mkdir ($rcs_dir, 0777) || die "Can't mkdir: $!";
  }
}

sub rcs_full_name {
  my ($self, $key) = @_;

  die 'No key' unless $key;

  unless ($^O =~ /Win32/) {
    return File::Spec
      ->catfile($self->{directory},
                'RCS',
                WiKicker::Tie::File::key_to_file_name($key) . ',v');
  }
  else {
    return File::Spec
      ->catfile($self->{directory},
                'RCS',
                WiKicker::Tie::File::key_to_file_name($key));
  }
}

sub _store {
  my ($self, $key, $value, $append) = @_;
  $self->exclusive_lock_key($key);
  my $file = $self->file_full_name($key);
  my $rcs  = $self->rcs_full_name($key);
  my $old_revision = $self->_control_old_revision($file, $rcs, $key);
  unless ($append) {
    if ($self->_write_file($file, $value)) {
      $self->_control_new_revision($file, $key, $rcs, $old_revision);
      $self->unlock_key($key);
    }
    else {
      $self->unlock_key($key);
      die "Can't store data to: $file";
    }
  }
  else {
    if ($self->_append_file($file, $value)) {
      $self->_control_new_revision($file, $key, $rcs, $old_revision);
      $self->unlock_key($key);
    }
    else {
      $self->unlock_key($key);
      die "Can't append data to: $file";
    }
  }
}

sub _delete {
  my ($self, $key) = @_;

  $self->exclusive_lock_key($key);

  $self->_check_in($self->file_full_name($key), $key);
  $self->_delete_work_file($key);

  $self->unlock_key($key);
}



sub _control_old_revision {
  my ($self, $file, $rcs, $key) = @_;

  # todo!
  #if ($self->lock_mode_key($key) != LOCK_EX) {
  #  die "No lock";
  #}

  unless (-e $file) {
    if (-e $rcs) {
      $self->_lock_rcs($file, $key);
      return $self->_head_revision_no($key);
    }
    return '';
  }
  my $st = stat($file) || die "No $file: $!";
  my $mtime = $st->mtime;
  my $now = time;
  if ($now > $mtime + $self->{options}->{check_in_interval}
     || $self->{options}->{force_check_in}) {
    $self->_check_in($file, $key);
    $self->_check_out($file, $key);
  }
  return '';
}

sub _control_new_revision {
  my ($self, $file, $key, $rcs, $old_revision) = @_;

  unless (-e $rcs) {
    # todo!
    #if ($self->lock_mode_key($key) != LOCK_EX) {
    #  die "No lock";
    #}

    $self->_check_in($file, $key);
    $self->_check_out($file, $key);
  }
  else {
    unless ($old_revision) {
      return;
    }
    if ($old_revision =~ /^(\d+)\.(\d+)$/) {
      my $major = $1;
      $major++;

      # !todo
      #if ($self->lock_mode_key($key) != LOCK_EX) {
      #  die "No lock";
      #}

      $self->_check_in($file, $key, "$major.1");
      $self->_check_out($file, $key);
    }
  }
}


sub exists_backup {
  my ($self, $key) = @_;

  return -e $self->rcs_full_name($key);
}


sub head_revision_no {
  my ($self, $key) = @_;

  die 'No key' unless $key;

  $self->shared_lock_key($key);
  my $revision_no;
  eval {
    $revision_no = $self->_head_revision_no($key);
  };
  if ($@) {
    my $message = $@;
    $self->unlock_key($key);
    die $message;
  }
  $self->unlock_key($key);
  return $revision_no;
}

sub latest_revision_no {
  my ($self, $key) = @_;

  die 'No key' unless $key;

  $self->shared_lock_key($key);
  my $head_revision_no;
  eval {
    $head_revision_no = $self->_head_revision_no($key);
  };
  if ($@) {
    my $message= $@;
    $self->unlock_key($key);
    die $message;
  }
  if ($head_revision_no =~ /^(\d+)\.1$/) {
    my $major = $1;
    my $file = $self->file_full_name($key);
    if (`rcsdiff -q -r$major.1 $file`) {
      $self->unlock_key($key);
      return "$major.2";
    }
    else {
      $self->unlock_key($key);
      return "$major.1";
    }
  }

  $self->unlock_key($key);
  $head_revision_no =~ /^(\d+)\.(\d+)$/;
  my $major = $1;
  my $minor = $2;
  $minor++;
  return "$major.$minor";
}

sub _head_revision_no {
  my ($self, $key) = @_;

  # todo!
  #if ($self->lock_mode_key($key) == 0) {
  #  die "No lock";
  #}

  my $file = $self->file_full_name($key);
  unless (open(RLOG, "rlog -h $file |")) {
    die "Can't execute rlog: $!";
  }

  while (<RLOG>) {
    chomp;
    if (/^head: (\d+\.\d+)/) {
      close(RLOG);
      return $1;
    }
  }

  close(RLOG);

  return 'unknown';
}


sub get_revision {
  my ($self, $key, $revision_no) = @_;

  die 'No key' unless $key;
  die 'No revision no' unless $revision_no;

  unless ($revision_no =~ /^\d+\.\d+$/) {
    die 'Invvalid revision no';
  }

  my $file = $self->file_full_name($key);
  $self->shared_lock_key($key);

  unless (open(REVISION, "co -ko -q -p$revision_no $file |")) {
    $self->unlock_key($key);
    die "Can't executer co: $!";
  }
  my $value;
  {
    local $/;
    $value = <REVISION>;
    close(REVISION);
  }
  $self->unlock_key($key);
  return $value;
}

sub get_head_revision {
  my ($self, $key) = @_;

  my $file = $self->file_full_name($key);
  $self->shared_lock_key($key);

  unless (open(REVISION, "co -ko -q -p $file |")) {
    $self->unlock_key($key);
    die "Can't executer co: $!";
  }
  my $value;
  {
    local $/;
    $value = <REVISION>;
    close(REVISION);
  }
  $self->unlock_key($key);
  return $value;
}

sub log {
  my ($self, $key) = @_;

  die 'No key' unless $key;

  my $file = $self->file_full_name($key);

  $self->shared_lock_key($key);

  unless (open(RLOG, "rlog $file |")) {
    $self->unlock_key($key);
    die "Can't execute rlog: $!";
  }

  my @result;

  my $revision;
  my $date;
  my $plus_lines;
  my $minus_lines;
  while (<RLOG>) {
    chomp;
    my $line = $_;
    if (/^revision\s+(\d+\.\d+)/) {
      $revision = $1;
      next;
    }
    if (m|^date:\s+(\d+)/(\d+)/(\d+)\s+(\d+):(\d+):(\d+);(.*)$|) {
      my ($year, $mon, $mday, $hour, $min, $sec, $rest)
        = ($1, $2, $3, $4, $5, $6, $7);
      $date = timegm($sec, $min, $hour, $mday, $mon - 1, $year);
      if (m|lines:\s+\+(\d+)\s+-(\d+)|) {
        $plus_lines = $1;
        $minus_lines = $2;
      }
      else {
        $plus_lines = -1;
        $minus_lines = -1;
      }
      push(@result, [ $revision, $date, $plus_lines, $minus_lines ]);
    }
  }

  return @result;
}

sub _check_in {
  my ($self, $file, $key, $new_revision) = @_;

  # todo!
  #if ($self->lock_mode_key($key) != LOCK_EX) {
  #  die "No lock";
  #}

  my $revision_option = '';
  if ($new_revision) {
    $revision_option = "-r$new_revision";
  }

  my $check_in_message = 'update';
  if (-e $file) {
    my $date = gmtime(stat($file)->mtime);
    my $command = qq(ci -m"$check_in_message" -t-"" -q -d"$date" $revision_option $file);
    #warn $command;
    if(`$command`) {
      die 'ci error!';
    }
  }
  else {
    if (`ci -m"$check_in_message" -t-"" -q $revision_option $file`) {
      die 'ci error!';
    }
  }
}

sub _check_out {
  my ($self, $file, $key) = @_;

  # todo!
  #if ($self->lock_mode_key($key) != LOCK_EX) {
  #  die "No lock";
  #}

  #`co -ko -l -q $file`;
  `rcs -U -q $file`;
  `rcs -u -M -q $file`;
  `co -ko -u -q $file`;
}

sub _lock_rcs {
  my ($self, $file, $key) = @_;

  # todo!
  #if ($self->lock_mode_key($key) != LOCK_EX) {
  #  die "No lock";
  #}
  `rcs -ko -l -q $file`;
}

1;
