#
# WiKicker
#
#  Factory.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Factory.pm,v 1.10 2005/02/26 07:44:15 naney Exp $
#
#

package WiKicker::WRI::Factory;

use strict;
use WiKicker::WRI::URN;
use WiKicker::WRI::URL;
use WiKicker::WRI::PWRN;
use WiKicker::WRI::UWRN;
use WiKicker::WRI::SWRN;
use WiKicker::WRI::RWRN;
use WiKicker::Loader;
use Log::Log4perl;

use Carp;

use vars qw(%default_registered_scheme);

BEGIN {
  %default_registered_scheme =
  (
   page           => 'WiKicker::WRI::Page',
   'page-subject' => 'WiKicker::WRI::PageSubject',
   'page-mtime'   => 'WiKicker::WRI::PageMtime',
   mtimelink      => 'WiKicker::WRI::PageMtime',
   image          => 'WiKicker::WRI::Image',
   limage         => 'WiKicker::WRI::Image',
   rimage         => 'WiKicker::WRI::Image',
   imagemap       => 'WiKicker::WRI::Imagemap',
   index          => 'WiKicker::WRI::Index',
   isbn           => 'WiKicker::WRI::ISBN',
   ISBN           => 'WiKicker::WRI::ISBN',
   asin           => 'WiKicker::WRI::ASIN',
   ASIN           => 'WiKicker::WRI::ASIN',
   rfc            => 'WiKicker::WRI::Rfc',
   RFC            => 'WiKicker::WRI::Rfc',
   gmtime         => 'WiKicker::WRI::Gmtime',
   datetime       => 'WiKicker::WRI::Datetime',
   search         => 'WiKicker::WRI::Search',
   href           => 'WiKicker::WRI::Href',
   form           => 'WiKicker::WRI::Form',
   interwiki      => 'WiKicker::WRI::RWRN',
   flickr         => 'WiKicker::WRI::Flickr',
   'l-flickr'     => 'WiKicker::WRI::Flickr',
   'r-flickr'     => 'WiKicker::WRI::Flickr',
  );
}

# WRI
#  +- URI
#  |   +- URL                          WRI::URL
#  |   |
#  |   +- URN                          WRI::URN
#  |
#  +- (WRN)
#      +- QWRN (qualified)
#      |    |
#      |    + RWRN registered scheme WRN     WRI::RWRN
#      |    |
#      |    +- UWRN unregistered scheme WRN  WRI::URWRN
#      |
#      +- NQWRN
#          |
#          +- PWRN PageReference (PageName + FragmentIdentifer) WRI::PRWRN
#          |
#          +- SWRN - Special WRN                                 WRI::SWRN
#                (and empty wrn)

sub new {
  my ($class, %param) = @_;

  my $self = bless {}, $class;

  if (defined $param{user_rwrn_map_object}) {
    $self->{user_rwrn_map_object} = $param{user_rwrn_map_object};
  }

  return $self;
}

sub create_wri {
  my ($self, $string) = @_;

  my ($scheme, $rest) = get_scheme($string);
  if (length($scheme) > 0) {
    return $self->_create_with_scheme($string, $scheme, $rest);
  }
  else {
    return _create_with_no_scheme($string);
  }
}

sub get_scheme {
  my $string = shift;

  return ('', '') unless defined $string;

  if ($string =~ /^((?:[^:\\]|(?:\\.))*?):(.*)$/o) {
    my $scheme = $1;
    my $rest = $2;
    $rest = '' unless defined($rest);
    return ($scheme, $rest);
  }
  else {
    return ('', $string);
  }
}

my %uri_scheme =
  (
   ftp    => 1,
   http   => 1,
   gopher => 1,
   mailto => 1,
   news => 1,
   nntp => 1,
   telnet => 1,
   wais => 1,
   file => 1,
   prospero => 1,
   'z39.50s' => 1,
   cid => 1,
   mid => 1,
   vemmi => 1,
   service => 1,
   imap => 1,
   nfs => 1,
   acap => 1,
   rtsp => 1,
   rtsp => 1,
   tip => 1,
   pop => 1,
   data => 1,
   dav => 1,
   opaquelocktoken => 1,
   sip => 1,
   tel => 1,
   fax => 1,
   modem => 1,
   ldap => 1,
   https => 1,
   'soap.beep' => 1,
   'soap.beeps' => 1,
   urn => 1,
   go => 1
  );

sub _create_with_scheme {
  my ($self, $string, $scheme, $rest) = @_;

  if (defined ($uri_scheme{$scheme})) {
    if ($scheme eq 'urn') {
      return WiKicker::WRI::URN->new($string, $scheme, $rest);
    }
    else {
      return WiKicker::WRI::URL->new($string, $scheme, $rest);
    }
  }
  else {
    return $self->_create_qwrn($string, $scheme, $rest);
  }
}

sub _create_with_no_scheme {
  my $string = shift;

  if (_is_swrn($string)) {
    return WiKicker::WRI::SWRN->new($string);
  }
  else {
    return WiKicker::WRI::PWRN->new($string);
  }
}


my %loaded_class;

sub _load_rwrn_class {
  my ($self, $class) = @_;

  if ($class ne 'WiKicker::WRI::RWRN' && !exists $loaded_class{$class}) {
    eval {
      WiKicker::Loader::load_module($class);
    };
    if ($@) {
      # already loaded?
      Log::Log4perl->get_logger->info("Can't load RWRN module $class: $@");
    }
    $loaded_class{$class} = 1;
  }
  return 1;
}

sub _user_rwrn_class_name {
  my ($self, $scheme) = @_;
  if (defined $self->{user_rwrn_map_object}) {
    return $self->{user_rwrn_map_object}->user_scheme_to_class_name($scheme);
  }
  return undef;
}

sub _create_qwrn {
  my ($self, $string, $scheme, $rest) = @_;

  # user rwrn
  my $class = $self->_user_rwrn_class_name($scheme);
  if (defined $class) {
    $self->_load_rwrn_class($class);
    my $rwrn;
    eval {
      $rwrn = $class->new($string, $scheme, $rest);
    };
    if ($@) {
      Log::Log4perl->get_logger
          ->error("Can't create user defined RWRN object $class: $@");
    }
    else {
      return $rwrn;
    }
  }

  # default rwrn
  if (defined $default_registered_scheme{$scheme}) {
    my $class = $default_registered_scheme{$scheme};
    $self->_load_rwrn_class($class);
    my $rwrn;
    eval {
      $rwrn = $class->new($string, $scheme, $rest);
    };
    if ($@) {
      die "Can't create RWRN object $class: $@";
    }
    return $rwrn
  }
  else {
    $scheme =~ s/\\(.)/$1/g;
    return WiKicker::WRI::UWRN->new($string, $scheme, $rest);
  }
}

sub _is_swrn {
  my $string = shift;

  if (!(defined $string) || length($string) == 0) {
    return 1;
  }

  if ($string =~ /^\s+$/) {
    return 1;
  }

  if ($string =~ /^(?:rfc|RFC)\d+$/) {
    return 1;
  }
  if ($string =~ /^(ISBN|isbn|ASIN|asin).+$/) {
    return 1;
  }
  return 0;
}

1;
