#
# WiKicker
#
#  RobotUserAgent.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: RobotUserAgent.pm,v 1.2 2004/08/03 12:55:52 naney Exp $
#
#

package WiKicker::WWW::Robot::RobotUserAgent;

use Storable;
use MLDBM;
use MLDBM::Sync::SDBM_File;
use LWP::RobotUA;
use HTTP::Date;
use WiKicker::WWW::Robot::RobotUriQueue;

sub new {
  my ($class, $agent, $from, $robot_rules_db_file_name) = @_;

  my $self = bless {agent => $agent,
                    from => $from,
                    robot_rules_db_file_name => $robot_rules_db_file_name,
                    queue => WiKicker::WWW::Robot::RobotUriQueue->new,
                   },
    $class;

  return $self;
}

sub request {
  my ($self, $uri, $last_modified) = @_;

  my $open_time = $self->{queue}->uri_open_time($uri);
  if (defined $open_time) {
    if ($open_time > time) {
      $self->add($uri, $open_time);
      return undef;
    }
  }

  my $request = HTTP::Request->new('GET', $uri);
  if (defined $last_modified) {
    $request->if_modified_since($last_modified);
  }

  my $response = $self->_user_agent->request($request);
  unless ($response->is_success) {
    if ($response->code eq HTTP::Status::RC_SERVICE_UNAVAILABLE) {
      my $retry_after = $response->header('Retry-After');
      if ($retry_after) {
        $self->add($uri, HTTP::Date::str2time($retry_after));
        return undef;
      }
      else {
        return $response;
      }
    }
  }
  return $response;
}

sub add {
  my ($self, $uri, $open_time) = @_;

  $self->{queue}->add($uri, $open_time);
}

sub delay {
  my ($self, $new_delay) = @_;

  return $self->_user_agent($new_delay);
}

sub print_queue {
  my ($self, $handle) = @_;
  $self->{queue}->print_queue($handle);
}

sub _user_agent {
  my $self = shift;

  if (exists $self->{user_agent}) {
    return $self->{user_agent};
  }

  my $rules = WiKicker::WWW::Robot::RobotRulesDatabase->new($self->{agent},
                                                      $self->{robot_rules_db_file_name});
  my $agent = LWP::RobotUA->new($self->{agent}, $self->{from}, $rules);
  $agent->use_sleep(0);
  $agent->delay(10);
  $agent->timeout(20);

  $self->{user_agent} = $agent;

  return $agent;
}

package WiKicker::WWW::Robot::RobotRulesDatabase;

use Storable;
use MLDBM;
use MLDBM::Sync::SDBM_File;
use WWW::RobotRules::AnyDBM_File;
use Fcntl;
use HTTP::Status;

use vars qw(@ISA);
BEGIN {
  @ISA = qw(WWW::RobotRules::AnyDBM_File);
}

# override AnyDBM_File 1.10
sub new
{
  my ($class, $ua, $file) = @_;
  Carp::croak('WiKicker::HTTP::RobotRulesDatabase filename required') unless $file;

  my $self = bless { }, $class;
  $self->{'filename'} = $file;
  $MLDBM::UseDB = 'MLDBM::Sync::SDBM_File';
  $MLDBM::Serializer = 'Storable';
  tie %{$self->{'dbm'}}, 'MLDBM', $file, O_CREAT|O_RDWR, 0640
    or Carp::croak("Can't open $file: $!");

  if ($ua) {
      $self->agent($ua);
  } else {
      # Try to obtain name from DBM file
      $ua = $self->{'dbm'}{'|ua-name|'};
      Carp::croak('No agent name specified') unless $ua;
  }

  $self;
}

sub agent {
    my($self, $newname) = @_;
    my $old = $self->{'dbm'}{'|ua-name|'};
    if (defined $newname) {
        $newname =~ s!/?\s*\d+.\d+\s*$!!;  # loose version
        unless ($old && $old eq $newname) {
        # Old info is now stale.
            my $file = $self->{'filename'};
            untie %{$self->{'dbm'}};
            $MLDBM::UseDB = 'MLDBM::Sync::SDBM_File';
            $MLDBM::Serializer = 'Storable';
            tie %{$self->{'dbm'}}, 'MLDBM', $file, O_TRUNC|O_RDWR, 0640;
            %{$self->{'dbm'}} = ();
            $self->{'dbm'}{'|ua-name|'} = $newname;
        }
    }
    $old;
}

1;
