#
# WiKicker
#
#  NormalPage.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: NormalPage.pm,v 1.12 2005/01/09 09:33:23 naney Exp $
#
#

package WiKicker::WikiCGI::Page::NormalPage;

use 5.005;
use strict;
use WiKicker::WikiCGI::Page;
use WiKicker::Loader;
use WiKicker::User;
use WiKicker::UTF8;
use AutoLoader 'AUTOLOAD';

use vars qw(@ISA);
@ISA = qw(WiKicker::WikiCGI::Page);

sub do_read {
  my $self = shift;

  if ($self->request->request_method eq 'GET') {
    unless ($self->is_modified) {
      return 'notmodified';
    }
  }

  if ($self->request->is_default_view_user
      && !($self->request->query->param('revision'))) {
    if ($self->_challenge_to_use_cache) {
      $self->request->attribute('editable', 1);
      $self->db->shared_lock_pages;
      return 'readcache';
    }
  }

  $self->set_http_last_modified_for_send;

  return $self->_do_read;
}

sub _do_read {
  my $self = shift;

  my $db = $self->db;
  my $name = $self->name;
  my $revision = $self->request->query->param('revision');

  if ($revision) {
    $self->request->attribute('revision', $revision);
    $self->request->attribute('editable', 0);
    $db->shared_lock_pages;
    $self->request->attribute('content', $db->get_revision($name, $revision));
    $self->request->attribute('normalpage', 1);
    return 'read';
  }
  else {
    if ($db->exists_live_page($name)) {
      return $self->_do_read_exists_page($db, $name);
    }
    else {
      return $self->_do_read_not_exists_page($db);
    }
  }

}

sub _do_read_exists_page {
  my ($self, $db, $name) = @_;

  $db->shared_lock_pages;

  $self->request->attribute('content', $db->page($name));
  $self->request->attribute('latestrevision', $db->page_revision_no($name));
  $self->request->attribute('editable', 1);

  $self->request->attribute('normalpage', 1);
  return 'read';
}

sub _do_read_not_exists_page {
  my ($self, $db) = @_;

  $self->request->attribute('editable', 1);
  $self->request->attribute('content', '');

  $db->shared_lock_pages;

  $self->request->attribute('normalpage', 1);
  return 'read';
}

sub _challenge_to_use_cache {
  my $self = shift;

  my $cache_key = $self->request->environment->html_fragment_cache_key($self->name);
  my $cached_data = $self->request->environment->get_cache_value($cache_key);
  my $last_modified = $self->request->environment->last_modified_for_view;
  unless (defined $cached_data) {
    $self->request->attribute('head_comments', [ 'no read cache' ]);
    return 0;
  }
  if ($cached_data->{t} >= $last_modified) {
    $self->request->attribute('cached_content', $cached_data->{c});
    $self->request->attribute('head_comments',
                              [ 'read cache hit! ' . $cached_data->{t} ]);
    return 1;
  }
  else {
    $self->request->attribute('head_comments',
                              [ 'read cache old! ' . $cached_data->{t} ]);
  }
  return 0;
}



sub do_rlog;
sub do_diff;
sub do_edit;
sub do_write;
sub do_copy;
sub do_delete;
sub _update_user_name_by_edit_form;
sub _do_write_commit;
sub do_comment;
sub _insert_comment;
sub is_cancel_request;
sub is_commit_request;
sub _is_valid_last_modified;
sub _write_page;
sub _parse_document;
sub _do_commit_processor;
#sub _is_dangling_name;
#sub _get_page_link;
sub _get_subject;
sub _update_interwiki_definition;
sub _export_page;
#sub do_TrackBack;


1;

__END__
sub do_rlog {
  my $self = shift;

  my $db = $self->db;

  $db->shared_lock_pages;
  my @rlog = $db->page_rlog($self->name);

  $self->request->attribute('rlog', \@rlog);

  return 'rlog';
}

sub do_diff {
  my  $self = shift;

  # from search engine?
  return 'redirect' if ($self->_check_redirect);

  my $db = $self->db;
  my $name = $self->name;
  WiKicker::Loader::load_module('WiKicker::Diff');
  $db->shared_lock_pages;
  my $result = WiKicker::Diff::make_diff($db->page($name), $db->get_head_revision($name));

  $self->request->attribute('diff', $result);

  return 'diff';
}

sub _set_read_redirect_attribute {
  my $self = shift;
  WiKicker::Loader::load_module('WiKicker::URI');

  $self->request->
    attribute('redirect',
              WiKicker::URI::make_read_page_url($self->request->url,
                                                $self->name));
}

sub _check_redirect {
  my $self = shift;

  WiKicker::Loader::load_module('WiKicker::HTTP::Referer');
  my $referer = WiKicker::HTTP::Referer->new;
  if ($referer->is_search_engine_result_page) {
    $self->_set_read_redirect_attribute;
    return 1;
  }
  return 0;
}

sub do_edit {
  my $self = shift;

  # from search engine?
  return 'redirect' if ($self->_check_redirect);

  my $db = $self->db;
  my $name = $self->name;
  $db->shared_lock_pages;
  my $wiki_page;
  my $section = $self->request->query->param('section');

  if (defined $section && length($section) > 0) {
    if ($db->exists_live_page($name)) {
      WiKicker::Loader::load_module("WiKicker::WikiPage::GetSectionRawVisitor");
      my $visitor = WiKicker::WikiPage::GetSectionRawVisitor->new;
      my $document_node = $self->_parse_document($db->page($name));

      $visitor->section_number($section);
      $visitor->visit($document_node);

      $wiki_page = $visitor->section;
    }
    else {
      $wiki_page = '';
    }
    $self->request->attribute('section', $section);
  }
  else {
    if ($db->exists_live_page($name)) {
      $wiki_page = $db->page($name);
    }
    else {
      $wiki_page = '';
    }
  }

  $self->request->attribute('content', $wiki_page);
  return 'edit';

}

sub do_write {
  my $self = shift;

  if ($self->is_cancel_request) {
    $self->request->attribute('action_label', 'read');
    return $self->_do_read;
  }

  # @TODO test!!!!!
  #if ($self->request->user->id eq 'aaaaaaaaaaaaaaaaaaa') {
  #  $self->_set_read_redirect_attribute;
  #  return 'redirect';
  #}

  if ($self->_get_properties('param.NormalPage.ignore.file')) {
    WiKicker::Loader::load_module('WiKicker::WikiCGI::IgnorePatternFile');
    my $ignore_patterns = WiKicker::WikiCGI::IgnorePatternFile->new(file => $self->_get_properties('param.NormalPage.ignore.file'));
    if ($ignore_patterns->match($self->request->param('content'))) {
      $self->_set_read_redirect_attribute;
      return 'redirect';
    }
  }

  my $db = $self->db;

  if ($self->is_commit_request) {
    $db->exclusive_lock_pages;
    unless ($self->_is_valid_last_modified) {
      $self->request->attribute('content', $self->request->param('content'));
      return 'conflict'
    }

    $self->_update_user_name_by_edit_form;

    my $section = $self->request->query->param('section');
    if (defined $section && length($section) > 0) {
      $self->request->attribute('section', $section);
    }

    return $self->_do_write_commit;
  }
  else {
    $db->shared_lock_pages;
    unless ($self->_is_valid_last_modified) {
      $self->request->attribute('content', $self->request->param('content'));
      return 'conflict'
    }

    $self->_update_user_name_by_edit_form;

    my $section = $self->request->query->param('section');
    if (defined $section && length($section) > 0) {
      $self->request->attribute('section', $section);
    }

    $self->request->attribute('content', $self->request->param('content'));
    $self->request->attribute('action_label', 'Preview');
    $self->db->shared_lock_pages;
    return 'preview';
  }
}

sub do_copy {
  my $self = shift;

  die 'No Name' unless $self->name;
  my $from = $self->request->param('from') || '';
  return 'error' unless $from;
  my $db = $self->db;
  $db->exclusive_lock_pages;
  unless ($db->exists_live_page($from)) {
    return 'error';
  }
  if ($db->exists_live_page($self->name)) {
    return 'error';
  }
  my $from_content = $db->page($from);
  $self->_write_page($self->name, $from_content);

  $self->mail($self->name, 'copy', $from_content);


  return 'success';
}

sub do_delete {
  my $self = shift;

  my $name = $self->name;
  die 'No WikiName' unless $name;

  if ($self->is_cancel_request) {
    $self->request->attribute('action_label', 'read');
    return $self->_do_read;
  }

  if ($self->is_commit_request) {
    my $db = $self->db;
    $db->exclusive_lock_pages;
    $db->delete_page($name);
    $self->request->attribute('successtype', 'delete');

    $self->mail($name, 'delete', '');

    return 'success';
  }

  return 'error';
}


sub _update_user_name_by_edit_form {
  my $self = shift;

  my $user = $self->request->user;
  if ($self->request->param('un')) {
    $user->name($self->request->param('un'));
  }
}

sub _do_write_commit {
  my $self = shift;

  my $name    = $self->name;
  die 'No WikiName' unless $name;
  my $db      = $self->db;
  my $content = $self->request->param('content');

  my $section = $self->request->query->param('section');
  if (defined $section && length($section) > 0) {
    WiKicker::Loader::load_module('WiKicker::WikiPage::ReplaceSectionVisitor');

    my $visitor = WiKicker::WikiPage::ReplaceSectionVisitor->new;
    my $document_node = $self->_parse_document($db->page($name));

    $visitor->section_number($section);
    $visitor->new_section($content);
    $visitor->visit($document_node);

    $content = $visitor->new_raw;
  }

  $content =~ s/(\r\n?)/\n/g;

  if ($content eq '') {
    if ($db->exists_live_page($name)) {
      return 'confirmdelete';
    }
    else {
      return 'success';
    }
  }

  # no change?
  my $old_wiki_page = $self->db->page($name);
  if ($old_wiki_page eq $content) {
    $self->_set_read_redirect_attribute;
    return 'redirect';
  }


  $self->_write_page($name, $content);

  $self->mail($name, 'write', $content);

  return 'success';
}

sub do_comment {
  my $self = shift;

  my $comment_name = $self->request->param('comment.name') || '';
  my $comment_number = $self->request->param('comment.number');
  my $comment_message = $self->request->param('comment.message');
  my $charset = lc($self->request->param('form.charset') || 'UTF-8');

  $comment_name = WiKicker::User::normalize_name($comment_name);
  $comment_message=~ s/\s+$//;

  unless ($comment_message) {
    #$self->request->attribute('action_label', 'read');
    #return $self->_do_read;
    $self->_set_read_redirect_attribute;
    return 'redirect';
  }

  # @TODO test!!!!!
  if ($self->request->user->id eq 'QJmb9j1z7UYAADQlSFs') {
    $self->_set_read_redirect_attribute;
    return 'redirect';
  }


  if ($charset eq 'euc' || $charset eq 'euc-jp') {
      $comment_message = WiKicker::UTF8::decode('euc', $comment_message);
      $comment_message = WiKicker::UTF8::decode('euc', $comment_name);
  }

  $self->db->exclusive_lock_pages;

  # check last post
  my $last_comment = $self->db->information($self->name, 'lastcomment');
  $last_comment = '' unless defined $last_comment;
  if ($last_comment eq $comment_message) {
    $self->_set_read_redirect_attribute;
    return 'redirect';
  }

  my $wiki_page = $self->db->page($self->name);


  if ($comment_name) {
    $self->request->user->name($comment_name);
  }
  elsif ($self->request->user->name) {
    $comment_name = $self->request->user->name;
  }

  my $document_node = $self->_parse_document($wiki_page);

  $self->_insert_comment($document_node, $comment_name,
                         $comment_number, $comment_message);

  my $new_page_content = $document_node->to_string;
  if ($wiki_page ne $new_page_content) {
    $self->_write_page($self->name, $new_page_content);
    $self->db->information($self->name, 'lastcomment', $comment_message);

    $self->mail($self->name,
                'comment',
                $comment_message . "\n----\n" . $new_page_content);

  }
  #return 'success';
  $self->_set_read_redirect_attribute;
  return 'redirect';
}


sub _insert_comment {
  my ($self, $document_node, $comment_name, $comment_number, $comment_message)
    = @_;

  WiKicker::Loader::load_module('WiKicker::WikiPage::InsertCommentVisitor');

  my $visitor = WiKicker::WikiPage::InsertCommentVisitor->new;
  $visitor->comment_name($comment_name);
  $visitor->comment_number($comment_number);
  $visitor->comment_message($comment_message);
  $visitor->comment_gmtime(time);
  $visitor->visit($document_node);
}

sub is_cancel_request {
  my $self = shift;

  if ($self->request->param('action.cancel')) {
    return 1;
  }
  else {
    return 0;
  }
}

sub is_commit_request {
  my $self = shift;

  if ($self->request->param('action.commit')) {
    return 1;
  }
  else {
    return 0;
  }
}

sub _is_valid_last_modified {
  my $self = shift;

  my $lastmodified = $self->request->param('lastmodified');
  if ($lastmodified eq $self->db->last_modified($self->name)) {
    return 1;
  }
  return 0;
}

sub _write_page {
  my ($self, $page_name, $page_content) = @_;

  my $db = $self->db;
  my $user_name = $self->request->user->name || '';

  my $document_node = $self->_parse_document($page_content);
  $self->_do_commit_processor($document_node);
  #$self->_get_page_link($document_node);
  my $subject = $self->_get_subject($document_node);

  $self->_export_page($page_name, $document_node);
  $self->_update_interwiki_definition($page_name, $document_node);
  $db->page($page_name, $document_node->to_string, $user_name, $subject);
  $db->delete_cache('dangling', $page_name);
  $self->request->environment->touch;
}


sub _parse_document {
  my ($self, $page_content) = @_;

  return $self->request->environment
    ->create_document_parser->parse($page_content);
}

sub _do_commit_processor {
  my ($self, $document_node) = @_;

  WiKicker::Loader::load_module('WiKicker::WikiPage::CommitProcessorVisitor');
  my $visitor = WiKicker::WikiPage::CommitProcessorVisitor->new(wri_factory => $self->request->environment->create_wri_factory);
  $visitor->resolver($self->request->environment->resolver);
  $visitor->visit($document_node);
}

# delete in 0.12
# sub _is_dangling_name {
#   my ($self, $name) = @_;

#   my $db = $self->db;
#   if ($db->exists_live_page($name)) {
#     return 0;
#   }

#   if ($self->request->configuration->is_pseudo_page($name)) {
#     return 0;
#   }

#   my @full_page_name_list = $db->full_page_name_list($name);
#   if (@full_page_name_list > 0) {
#     return 0;
#   }
#   return 1;
# }

# delete in 0.12
# sub _get_page_link {
#   my ($self, $document_node) = @_;

#   WiKicker::Loader::load_module("WiKicker::WikiPage::GetPageLinkVisitor");
#   my $get_page_link_visitor = WiKicker::WikiPage::GetPageLinkVisitor->new;
#   $get_page_link_visitor->visit($document_node);

#   my @page_name_list = $get_page_link_visitor->page_name_list;
#   my $db = $self->db();

#   my %page_name_hash;
#   map { $page_name_hash{$_}++ } @page_name_list;
#   for my $page_name (keys %page_name_hash) {
#     if ($page_name =~ /^(.*)#([^#]+)$/) {
#       $page_name = $1;
#     }
#     next unless $page_name;
#     $page_name = WiKicker::PageName::normalize($page_name);

#     my $dangling = $self->_is_dangling_name($page_name);
#     if ($dangling) {
#       $db->cache('dangling', $page_name, 1);
#     }
#     else {
#       $db->delete_cache('dangling', $page_name);
#     }
#   }
# }

sub _get_subject {
  my ($self, $document_node) = @_;

  WiKicker::Loader::load_module('WiKicker::WikiPage::GetSubjectVisitor');
  my $visitor = WiKicker::WikiPage::GetSubjectVisitor->new(wri_factory => $self->request->environment->create_wri_factory);
  $visitor->visit($document_node);

  return $visitor->subject(40); # @todo customize size
}

sub _update_interwiki_definition {
  my ($self, $page_name, $document_node) = @_;

  if ($page_name ne 'InterWikiDefinition') {
    return;
  }

  WiKicker::Loader::load_module('WiKicker::WikiPage::GetInterWikiDefinitionVisitor');
  my $visitor = WiKicker::WikiPage::GetInterWikiDefinitionVisitor->new;
  $visitor->visit($document_node);

  my $db = $self->db;

  $db->clear_cache('interwiki');
  my @definitions = $visitor->inter_wiki_definitions;
  for my $definition (@definitions) {
    $db->cache('interwiki', $definition->nid, $definition->nss);
  }
}

sub _export_page {
  my ($self, $page_name, $document_node) = @_;

  WiKicker::Loader::load_module('WiKicker::CGI::Export');
  WiKicker::CGI::Export::export_page($page_name,
                                     $document_node,
                                     $self->request);
}

# @deprecated
#sub do_TrackBack {
sub _do_TrackBack {
  eval "use Storable";

  my $self = shift;

  my $db = $self->db;
  my $mode = $self->request->param('__mode');
  if ($mode eq 'rss') {
    # @todo implement
    $db->shared_lock_pages;
    $self->request->attribute('message', 'Not implemented');
    return 'trackbackerror';
  }
  else {
    my $url       = $self->request->param('url') || '';
    my $excerpt   = $self->request->param('excerpt') || '';
    my $title     = $self->request->param('title') || '';
    my $blog_name = $self->request->param('blog_name') || '';
    my $timestamp = time;

    my $charset = $self->request->param('charset')
      || $self->request->param('ie') || '';
    if ($charset) {
      $charset = lc($charset);
      my $jcode_in = '';
      if ($charset eq 'euc' || $charset eq 'euc-jp') {
        $jcode_in = 'euc';
      }
      elsif ($charset eq 'x-sjis' || $charset eq 'sjis' || $charset eq 'shift_jis') {
        $jcode_in = 'sjis';
      }
      elsif ($charset eq 'iso-2022-jp' || $charset eq 'jis') {
        $jcode_in = 'jis';
      }
      if ($jcode_in) {
        $excerpt = WiKicker::UTF8::decode($jcode_in, $excerpt);
        $title = WiKicker::UTF8::decode($jcode_in, $title);
        $blog_name = WiKicker::UTF8::decode($jcode_in, $blog_name);
      }
    }

    unless ($url) {
      $db->shared_lock_pages;
      $self->request->attribute('message', 'No url');
      return 'trackbackerror';
    }

    WiKicker::Loader::load_module('WiKicker::TrackBack::Entry');
    WiKicker::Loader::load_module('WiKicker::TrackBack::EntryMap');

    my $entry = WiKicker::TrackBack::Entry->new;
    $entry->url($url);
    $entry->excerpt($excerpt);
    $entry->title($title);
    $entry->blog_name($blog_name);
    $entry->timestamp($timestamp);


    my $entry_map = $db->cache('trackback', $self->name);
    if (defined $entry_map) {
      $entry_map = Storable::thaw($entry_map);
    }
    else {
      $entry_map = WiKicker::TrackBack::EntryMap->new;
    }
    $entry_map->set($entry);
    $db->cache('trackback', $self->name, Storable::freeze($entry_map));

    $self->mail($self->name,
                'trackback',
                'url: ' . $entry->url . "\n" .
                'excerpt: ' . $entry->excerpt . "\n" .
                'title: ' . $entry->title . "\n" .
                'blog_name: ' . $entry->blog_name . "\n");

    $db->shared_lock_pages;
    return 'trackbackpingsuccess';
  }
}
