#
# WiKicker
#
#  HtmlTocVisitor.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: HtmlTocVisitor.pm,v 1.4 2004/08/03 12:55:52 naney Exp $
#
#
package WiKicker::WikiPage::HtmlTocVisitor;

use strict;
use WiKicker::HTML;
use WiKicker::WikiPage::TraverseVisitor;
use vars qw(@ISA);
@ISA = qw(WiKicker::WikiPage::TraverseVisitor);

sub size {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{size} = $value;
  }
  return $self->{size};
}

sub new {
  my $class = shift;

  my $root = WiKicker::WikiPage::HtmlTocVisitor::List->new;
  my $self = bless { anchor => 0,
                     root => $root,
                   }, $class;
  $self->size(0);
  return $self;
}

sub visit_HeadingNode {
  my ($self, $heading_node) = @_;

  my $level = $heading_node->level();
  $self->{anchor}++;
  my $anchor_name = $self->{anchor};
  my $string = '';
  for my $node ($heading_node->nodes) {
    $string .= $node->to_unarmor_string;
  }

  $self->_add_content($level, $string, $anchor_name);
  $self->size($self->size + 1);
}

sub _add_content {
  my ($self, $level, $string, $anchor_name) = @_;

  my $content = WiKicker::HTML::escape($string);
  chomp $content;
  $content =~ s/^\s+//;
  $self->{root}->add_item($level, qq(<a href="#h$anchor_name">$content</a>));
}

sub content {
  my $self = shift;

  return $self->{root}->to_html(0);
}

sub visit_ParagraphNode {
  my ($self, $paragraph_node) = @_;

  # ignore
}

sub visit_TableNode {
  my ($self, $table_node) = @_;

  # ignore
}

sub visit_TableRecordNode {
  my ($self, $record_node) = @_;

  # ignore
}

sub visit_TableCellNode {
  my ($self, $cell_node) = @_;

  # ignore
}

sub visit_ListNode {
  my ($self, $list_node) = @_;

  # ignore
}

sub visit_DlNode {
  my ($self, $dl_node) = @_;

  # ignore
}

sub visit_QuoteNode {
  my ($self, $quote_node) = @_;

  # ignore
}

sub visit_ListItemNode {
  my ($self, $li_node) = @_;

  # ignore
}

sub visit_DtNode {
  my ($self, $dt_node) = @_;

  # ignore
}

sub visit_DdNode {
  my ($self, $dd_node) = @_;

  # ignore
}

sub visit_QuoteLineNode {
  my ($self, $line_node) = @_;

  # ignore
}


package WiKicker::WikiPage::HtmlTocVisitor::List;

sub new {
  my $class = shift;

  my $self = bless { child => [] }, $class;
  return $self;
}

sub _add {
  my ($self, $new_node) = @_;

  push(@{$self->{child}}, $new_node);
}

sub last_item {
  my $self = shift;

  if (@{$self->{child}} > 0) {
    return $self->{child}->[-1];
  }
  else {
    return undef;
  }
}

sub add_item {
  my ($self, $level, $content) = @_;

  if ($level == 1) {
    my $item = WiKicker::WikiPage::HtmlTocVisitor::Item->new;
    $item->data($content);
    $self->_add($item);
  }
  else {
    my $last_item = $self->last_item;
    unless (defined $last_item) {
      $last_item = WiKicker::WikiPage::HtmlTocVisitor::Item->new;
      $self->_add($last_item);
    }
    $last_item->add_item($level - 1, $content);
  }
}

sub to_html {
  my ($self, $indent) = @_;

  if (@{$self->{child}} == 0) {
    return '';
  }

  my $result = ' ' x $indent . qq(<ol>\n);
  for my $child (@{$self->{child}}) {
    $result .= $child->to_html($indent + 2);
  }
  $result .= ' ' x $indent . qq(</ol>\n);

  return $result;
}

package WiKicker::WikiPage::HtmlTocVisitor::Item;

sub new {
  my $class = shift;

  my $self = bless { data => '', sub_list => undef }, $class;
  return $self;
}

sub data {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{data} = $value;
  }
  return $self->{data};
}

sub sub_list {
  my $self = shift;

  unless (defined $self->{sub_list}) {
    $self->{sub_list} = WiKicker::WikiPage::HtmlTocVisitor::List->new;
  }
  return $self->{sub_list};
}

sub add_item {
  my ($self, $level, $content) = @_;

  $self->sub_list->add_item($level, $content);
}

sub to_html {
  my ($self, $indent) = @_;

  my $result = ' ' x $indent . '<li>';
  $result .= $self->data;
  if (defined $self->{sub_list}) {
    $result .= "\n" . $self->{sub_list}->to_html($indent + 2);
    $result .= ' ' x $indent ;
  }
  $result .= qq(</li>\n);
}

1;
