#
# WiKicker
#
#  TextFormatVisitor.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: TextFormatVisitor.pm,v 1.4 2004/02/11 07:00:14 naney Exp $
#
#

package WiKicker::WikiPage::TextFormatVisitor;

use strict;
use WiKicker::UTF8;
use WiKicker::WikiPage::TraverseVisitor;

use vars qw(@ISA);
@ISA = qw(WiKicker::WikiPage::TraverseVisitor);

sub new {
  my $class = shift;

  my $self = bless { content => '',
                   }, $class;

  $self->_indent(0);
  return $self;
}

sub user {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{user} = $value;
  }
  return $self->{user};
}

sub _indent {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{_indent} = $value;
  }
  return $self->{_indent};
}

sub content {
  my $self = shift;

  return $self->{content};
}

sub visit_DocumentNode {
  my ($self, $document_node) = @_;
  
  for my $node ($document_node->nodes) {
    $self->{content} .= $node->accept($self);
  }
}

sub visit_ParagraphNode {
  my ($self, $paragraph_node) = @_;

  my $text = '';
  for my $node ($paragraph_node->nodes) {
    $text .= $node->accept($self);
  }
  $text =~ s/\n/ /g;
  return _fill_paragraph($text, 72, '', '');
}

sub visit_EmphasisNode {
  my ($self, $emphasis_node) = @_;
  my $text = '';
  for my $node ($emphasis_node->nodes) {
    $text .= $node->accept($self);
  }
  return $text;
}

sub visit_InlineBlockNode {
  my ($self, $inline_block_node) = @_;
  my $text = '';
  for my $node ($inline_block_node->nodes) {
    $text .= $node->accept($self);
  }
  return $text;
}

sub visit_HeadingNode {
  my ($self, $heading_node) = @_;

  my $text = '';
  for my $node ($heading_node->nodes) {
    $text .= $node->accept($self);
  }

  $text =~ s/^\s+//;
  $text =~ s/\s+$//;
  
  my $line = '-';
  if ($heading_node->level == 1) {
    $line = '*';
  }
  elsif ($heading_node->level == 2) {
    $line = '=';
  }

  my $heading_text = _fill_paragraph($text, 72, '', '');
  my $max_length = _get_max_width($heading_text);
  return "\n" . $heading_text . "\n" . ($line x $max_length) . "\n";
}  

sub visit_PreNode {
  my ($self, $pre_node) = @_;

  return $pre_node->to_string;
}

sub visit_TextNode {
  my ($self, $text_node) = @_;

  return $text_node->to_string;
}

sub visit_VerbatimNode {
  my ($self, $node) = @_;

  return $node->text;
}

sub visit_WriNode {
  my ($self, $wri_node) = @_;

  return $wri_node->original;
}

sub visit_BracketNameNode {
  my ($self, $bracket_name_node) = @_;

  if ($bracket_name_node->original eq '[[]]') {
    return '[[]]';
  }
  elsif ($bracket_name_node->content) {
    return $bracket_name_node->content;
  }
  else {
    return $bracket_name_node->wri->to_string;
  }
}

sub visit_HrNode {
  my ($self, $hr_node) = @_;

  return '-' x 76 . "\n";
}

sub visit_EmptyLinesNode {
  return "\n";
}

sub visit_ListNode {
  my ($self, $list_node) = @_;

  my $result;
  for my $node ($list_node->nodes) {
    $result .= $node->accept($self);
  }
  return $result;
}  

sub visit_DlNode {
  my ($self, $dl_node) = @_;

  my $result ;
  for my $node ($dl_node->nodes) {
    $result .= $node->accept($self);
  }
  return $result;
}

sub visit_QuoteNode {
  my ($self, $node) = @_;

  my $result;
  for my $child ($node->nodes) {
    if (ref $child eq 'WiKicker::WikiPage::QuoteNode') {
      $result .= $child->accept($child);
    }
    else {
      $result .= _fill_paragraph($result, 72, '>');
    }
  }
  return $result;
}

sub visit_ListItemNode {
  my ($self, $node) = @_;

  my $result;
  my $buffer;
  for my $child ($node->nodes) {
    if (ref $child eq 'WiKicker::WikiPage::ListNode') {
      $buffer =~ s/^\s+//;
      $buffer = ' ' . $buffer;
      $result .= _fill_paragraph($buffer, 72,
                                 ' ' x ($node->level + 1),
                                 '-' x $node->level);
      $buffer = '';
      $result .= $child->accept($self);
    }
    else {
      $buffer .= $child->accept($self);
    }
  }
  if ($buffer) {
    $buffer =~ s/^\s+//;
    $buffer = ' ' . $buffer;
    $result .= _fill_paragraph($buffer, 72,
                               ' ' x ($node->level + 1),
                               '-' x $node->level);
  }    
  return $result;
}

sub visit_DtNode {
  my ($self, $node) = @_;

  my $result;
  for my $child ($node->nodes) {
    $result .= $child->accept($self);
  }
  return _fill_paragraph($result, 72, '-');
}  

sub visit_DdNode {
  my ($self, $node) = @_;

  my $result;
  for my $child ($node->nodes) {
    $result .= $child->accept($self);
  }
  return _fill_paragraph($result, 72, '-');
}  

sub visit_QuoteLineNode {
  my ($self, $line_node) = @_;
  my $result;
  for my $node ($line_node->nodes) {
    $result .= $node->accept($self);
  }
  return $result;
}

  
sub _split_char {
  my $string = shift;

  my $length = WiKicker::UTF8::length_utf8($string);
  my @result = ();
  for (my $i = 0; $i < $length; $i++) {
    push(@result, WiKicker::UTF8::substr_utf8($string, $i, 1));
  }
  return @result;
}

sub _char_font_length {
  my $char = shift;

  if (length($char) == 1) {
    return 1;
  }
  else {
    return 2;
  }
}

sub _fill_paragraph {
  my ($text, $width, $prefix, $first_line_prefix) = @_;
  
  $prefix = '' unless defined $prefix;
  $first_line_prefix = $prefix unless defined $first_line_prefix;

  my @text_char = _split_char($text);
  my @lines;
  my $current_line = '';
  my $current_line_length = 0;
  for my $char (@text_char) {
    if ($current_line_length >= $width) {
      push(@lines, $current_line);
      $current_line = '';
      $current_line_length = 0;
    }
    my $char_length = _char_font_length($char);
    $current_line .= $char;
    $current_line_length += $char_length;
  }
  push(@lines, $current_line);

  if (@lines > 0) {
    $lines[0] = $first_line_prefix . $lines[0];
  }
  for (my $i = 1; $i < @lines; $i++) {
    $lines[$i] = $prefix . $lines[$i];
  }

  return join("\n", @lines);
}
  
sub _get_max_width {
  my $text = shift;

  my $result = 0;

  my @lines = split(/\n/, $text);
  for my $line (@lines) {
    my @line_char = _split_char($text);
    my $length = 0;
    map { $length += _char_font_length($_) } @line_char;
    if ($length > $result) {
      $result = $length;
    }
  }

  return $result;
}

1;
